/////////////////////////////////////////////////////////////////////////////
// Name:        richtextcontainer.h
// Purpose:     Manages a rich text control and suitable toolbars.
//              Handles standard rich text editing commands.
// Author:      Julian Smart
// Modified by:
// Created:     2006-11-30
// RCS-ID:      $Id: richtextcontainer.h,v 1.3 2007/06/16 23:42:17 anthemion Exp $
// Copyright:   (c) Julian Smart and Anthemion Software
// Licence:
/////////////////////////////////////////////////////////////////////////////

#ifndef _ANTHEMION_RICHTEXTCONTAINER_H_
#define _ANTHEMION_RICHTEXTCONTAINER_H_

#include "wx/richtext/richtextctrl.h"
#include "wx/richtext/richtextstyles.h"

#include "editorcontainer.h"
#include "toolbarcust.h"

/*
 * Standard rich text editor commands
 */

enum
{
    wxID_RICHTEXT_LOWEST = 7000,

    wxID_RICHTEXT_LEFT_ALIGN,
    wxID_RICHTEXT_CENTRE_ALIGN,
    wxID_RICHTEXT_RIGHT_ALIGN,
    wxID_RICHTEXT_JUSTIFY,

    wxID_RICHTEXT_TEXT_FONT,
    wxID_RICHTEXT_TEXT_SIZE,
    wxID_RICHTEXT_TEXT_COLOUR,
    wxID_RICHTEXT_TEXT_PROPERTIES,
    wxID_RICHTEXT_STYLE,
    wxID_RICHTEXT_PARAGRAPH_PROPERTIES,
    wxID_RICHTEXT_BULLETS_AND_NUMBERING,

    wxID_RICHTEXT_BOLD,
    wxID_RICHTEXT_ITALIC,
    wxID_RICHTEXT_UNDERLINE,
    wxID_RICHTEXT_NUMBERED_LIST,
    wxID_RICHTEXT_BULLETED_LIST,

    wxID_RICHTEXT_INSERT_SYMBOL,
    wxID_RICHTEXT_INSERT_PICTURE,
    wxID_RICHTEXT_INSERT_URL,

    wxID_RICHTEXT_CHOOSE_STYLE,
    wxID_RICHTEXT_EDIT_STYLES,

    wxID_RICHTEXT_INDENT_MORE,
    wxID_RICHTEXT_INDENT_LESS,
    wxID_RICHTEXT_LINESPACING_SINGLE,
    wxID_RICHTEXT_LINESPACING_HALF,
    wxID_RICHTEXT_LINESPACING_DOUBLE,

    wxID_RICHTEXT_HIGHEST
};

/*!
 * The rich text editor container
 */

class wxRichTextEditorContainerWindow: public wxEditorContainerWindow
{
 public:
    /// Ctors
    wxRichTextEditorContainerWindow() { Init(); }
    wxRichTextEditorContainerWindow(wxWindow *parent, wxWindowID id = wxID_ANY,
        const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0, int extraStyle = 0);
    ~wxRichTextEditorContainerWindow();

    bool Create(wxWindow *parent, wxWindowID id = wxID_ANY,
        const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0, int extraStyle = 0);

    void Init();

//// Event handlers

    void OnBold(wxCommandEvent& event);
    void OnItalic(wxCommandEvent& event);
    void OnUnderline(wxCommandEvent& event);

    void OnAlignLeft(wxCommandEvent& event);
    void OnAlignRight(wxCommandEvent& event);
    void OnAlignCentre(wxCommandEvent& event);

    void OnIndentLess(wxCommandEvent& event);
    void OnIndentMore(wxCommandEvent& event);

    void OnSingleLineSpacing(wxCommandEvent& event);
    void OnHalfLineSpacing(wxCommandEvent& event);
    void OnDoubleLineSpacing(wxCommandEvent& event);

    void OnTextColour(wxCommandEvent& event);
    void OnTextProperties(wxCommandEvent& event);
    void OnSelectTextFont(wxCommandEvent& event);
    void OnSelectTextSize(wxCommandEvent& event);
    void OnParagraphProperties(wxCommandEvent& event);

    void OnBulletedList(wxCommandEvent& event);
    void OnNumberedList(wxCommandEvent& event);

    void OnInsertSymbol(wxCommandEvent& event);
    void OnInsertURL(wxCommandEvent& event);
    void OnInsertPicture(wxCommandEvent& event);
    void OnBulletsAndNumbering(wxCommandEvent& event);

    void OnChooseStyle(wxCommandEvent& event);
    void OnEditStyles(wxCommandEvent& event);

    void OnRedirectCommand(wxCommandEvent& event);

/// Update event handlers

    void OnUpdateBold(wxUpdateUIEvent& event);
    void OnUpdateItalic(wxUpdateUIEvent& event);
    void OnUpdateUnderline(wxUpdateUIEvent& event);

    void OnUpdateAlignLeft(wxUpdateUIEvent& event);
    void OnUpdateAlignRight(wxUpdateUIEvent& event);
    void OnUpdateAlignCentre(wxUpdateUIEvent& event);

    void OnUpdateIndentLess(wxUpdateUIEvent& event);
    void OnUpdateIndentMore(wxUpdateUIEvent& event);

    void OnUpdateSingleLineSpacing(wxUpdateUIEvent& event);
    void OnUpdateHalfLineSpacing(wxUpdateUIEvent& event);
    void OnUpdateDoubleLineSpacing(wxUpdateUIEvent& event);

    void OnUpdateTextColour(wxUpdateUIEvent& event);
    void OnUpdateTextProperties(wxUpdateUIEvent& event);
    void OnUpdateParagraphProperties(wxUpdateUIEvent& event);

    void OnUpdateBulletedList(wxUpdateUIEvent& event);
    void OnUpdateNumberedList(wxUpdateUIEvent& event);

    void OnUpdateInsertSymbol(wxUpdateUIEvent& event);
    void OnUpdateInsertURL(wxUpdateUIEvent& event);
    void OnUpdateInsertPicture(wxUpdateUIEvent& event);
    void OnUpdateBulletsAndNumbering(wxUpdateUIEvent& event);

    void OnUpdateTextFont(wxUpdateUIEvent& event);
    void OnUpdateTextSize(wxUpdateUIEvent& event);

    void OnUpdateChooseStyle(wxUpdateUIEvent& event);
    void OnUpdateEditStyles(wxUpdateUIEvent& event);

    void OnUpdateRedirectCommand(wxUpdateUIEvent& event);

//// Accessors

    /// Set/get toolbar
    void SetToolBar(wxToolBarBase* toolbar) { m_toolBar = toolbar; }
    wxToolBarBase* GetToolBar() const { return m_toolBar; }

    /// Set/get rich text control
    void SetRichTextCtrl(wxRichTextCtrl* richTextCtrl) { m_richTextCtrl = richTextCtrl; }
    wxRichTextCtrl* GetRichTextCtrl() const { return m_richTextCtrl; }

    /// Set the style sheet and update the style combobox
    void SetStyleSheet(wxRichTextStyleSheet* sheet);

    /// Get a suitable top-level parent window given a subwindow
    wxWindow* GetParentWindow(wxWindow* win) const;

    /// Set/get editor enabled flag
    void SetEditorEnabled(bool enabled) { m_editorEnabled = enabled; }
    bool GetEditorEnabled() const { return m_editorEnabled; }

    /// Is a bulleted list selected?
    bool IsSelectionBulletList() const;

    /// Is a numbered list selected?
    bool IsSelectionNumberedList() const;

/// Operations

    /// Recreate the toolbars with the current settings
    virtual void RefreshToolbar(wxToolBarCustomisationInfo* info);

    /// Show or hide the toolbar
    virtual void UpdateVisibility(wxToolBarCustomisationInfo* info);

    /// Create standard customisation
    static bool CreateCustomisation(wxToolBarCustomisationInfo* info);

    /// Create and add standard toolbar. If already exists, delete tools.
    virtual bool CreateStandardToolBar(wxToolBarCustomisationInfo* info, wxToolBarBase* toolBar = NULL);

    /// Create standard toolbar items
    virtual bool CreateTools(wxToolBarBase* toolBar, wxToolBarCustomisationInfo* info);

    /// Create a control: override to create further controls
    virtual bool CreateControl(wxToolBarBase* toolBar, wxToolBarCustomisationInfo* info, const wxToolBarCustomisationInfoItem& item);

    /// Override to be notified when style sheet changed
    virtual void OnStyleSheetChanged(wxRichTextStyleSheet* WXUNUSED(sheet)) {}

    /// Override to set/get directory for image browsing
    virtual void SetImageDir(const wxString& dir);
    virtual wxString GetImageDir() const;

    /// Override to set/get image browser size and position
    virtual void SetImageBrowserRect(const wxRect& rect);
    virtual wxRect GetImageBrowserRect() const;

//// Implementation

    DECLARE_CLASS(wxRichTextEditorContainerWindow)
    DECLARE_EVENT_TABLE()

protected:
    wxToolBarBase*              m_toolBar;
    wxRichTextCtrl*             m_richTextCtrl;
    wxRichTextStyleComboCtrl*   m_styleComboCtrl;
    wxComboBox*                 m_pointSizeComboBox;
    wxComboBox*                 m_facenameComboBox;
    bool                        m_editorEnabled;
    static wxString             sm_imageDir;
    static wxRect               sm_imageBrowserRect;
};

#endif
    // _ANTHEMION_RICHTEXTCONTAINER_H_

